;--------------------------------------------------------------------------
;
;  Filename:     lpRegs.inc
;
;  Description:  Defines CYRF6936 registers
;
;--------------------------------------------------------------------------
; WirelessUSB LP Radio Driver Version 1.1
;--------------------------------------------------------------------------
;
; Copyright 2005-2006, Cypress Semiconductor Corporation.
;
; This software is owned by Cypress Semiconductor Corporation (Cypress)
; and is protected by and subject to worldwide patent protection (United
; States and foreign), United States copyright laws and international
; treaty provisions. Cypress hereby grants to licensee a personal,
; non-exclusive, non-transferable license to copy, use, modify, create
; derivative works of, and compile the Cypress Source Code and derivative
; works for the sole purpose of creating custom software in support of
; licensee product to be used only in conjunction with a Cypress integrated
; circuit as specified in the applicable agreement. Any reproduction,
; modification, translation, compilation, or representation of this
; software except as specified above is prohibited without the express
; written permission of Cypress.
;
; Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
; WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
; Cypress reserves the right to make changes without further notice to the
; materials described herein. Cypress does not assume any liability arising
; out of the application or use of any product or circuit described herein.
; Cypress does not authorize its products for use as critical components in
; life-support systems where a malfunction or failure may reasonably be
; expected to result in significant injury to the user. The inclusion of
; Cypress' product in a life-support systems application implies that the
; manufacturer assumes all risk of such use and in doing so indemnifies
; Cypress against all charges.
;
; Use may be limited by and subject to the applicable Cypress software
; license agreement.
;
;--------------------------------------------------------------------------


; -------------------------------
; Channel register
; -------------------------------
CHANNEL_ADR:                                    equ 0x00
CHANNEL_RST:                                    equ 0x48
CHANNEL_MSK:                                    equ 0x7f

CHANNEL_MAX:                                    equ 0x62
CHANNEL_MIN:                                    equ 0x00
CHANNEL_2P498_GHZ:                              equ 0x62
CHANNEL_2P4_GHZ:                                equ 0x00


; -------------------------------
; TX Length register
; -------------------------------
TX_LENGTH_ADR:                                  equ 0x01
TX_LENGTH_RST:                                  equ 0x00
TX_LENGTH_MSK:                                  equ 0xff


; -------------------------------
; TX Control register
; -------------------------------
TX_CTRL_ADR:                                    equ 0x02
TX_CTRL_RST:                                    equ 0x03

; See TX_IRQ for remaining bit position definitions

; TX_CTRL bit masks
TX_GO:                                          equ 0x80
TX_CLR:                                         equ 0x40


; -------------------------------
; TX Configuration register
; -------------------------------
TX_CFG_ADR:                                     equ 0x03
TX_CFG_RST:                                     equ 0x05

; separate bit field masks
TX_DATCODE_LEN_MSK:                             equ 0x20
TX_DATMODE_MSK:                                 equ 0x18
PA_VAL_MSK:                                     equ 0x07

; DATCODE_LEN register masks
DATCODE_LEN_64:                                 equ 0x20
DATCODE_LEN_32:                                 equ 0x00

; DATMODE register masks
DATMODE_1MBPS:                                  equ 0x00
DATMODE_8DR:                                    equ 0x08
DATMODE_DDR:                                    equ 0x10
DATMODE_SDR:                                    equ 0x18

; PA_SET register masks
PA_N30_DBM:                                     equ 0x00
PA_N25_DBM:                                     equ 0x01
PA_N20_DBM:                                     equ 0x02
PA_N15_DBM:                                     equ 0x03
PA_N10_DBM:                                     equ 0x04
PA_N5_DBM:                                      equ 0x05
PA_0_DBM:                                       equ 0x06
PA_4_DBM:                                       equ 0x07


; -------------------------------
; TX IRQ Status register
; -------------------------------
TX_IRQ_STATUS_ADR:                              equ 0x04

; TX_IRQ bit masks
XS_IRQ:                                         equ 0x80
LV_IRQ:                                         equ 0x40
TXB15_IRQ:                                      equ 0x20
TXB8_IRQ:                                       equ 0x10
TXB0_IRQ:                                       equ 0x08
TXBERR_IRQ:                                     equ 0x04
TXC_IRQ:                                        equ 0x02
TXE_IRQ:                                        equ 0x01


; -------------------------------
; RX Control register
; -------------------------------
RX_CTRL_ADR:                                    equ 0x05
RX_CTRL_RST:                                    equ 0x07

; See RX_IRQ register for bit positions definitions also used for this register

; RX_CTRL bit masks
RX_GO:                                          equ 0x80


; -------------------------------
; RX Configuration register
; -------------------------------
RX_CFG_ADR:                                     equ 0x06
RX_CFG_RST:                                     equ 0x92

AUTO_AGC_EN:                                    equ 0x80
LNA_EN:                                         equ 0x40
ATT_EN:                                         equ 0x20
HI:                                             equ 0x10
LO:                                             equ 0x00
FASTTURN_EN:                                    equ 0x08
RXOW_EN:                                        equ 0x02
VLD_EN:                                         equ 0x01


; -------------------------------
; RX IRQ register
; -------------------------------
RX_IRQ_STATUS_ADR:                              equ 0x07
; There is no default value for this register.

; RX_IRQ bit masks
RXOW_IRQ:                                       equ 0x80
SOFDET_IRQ:                                     equ 0x40
RXB16_IRQ:                                      equ 0x20
RXB8_IRQ:                                       equ 0x10
RXB1_IRQ:                                       equ 0x08
RXBERR_IRQ:                                     equ 0x04
RXC_IRQ:                                        equ 0x02
RXE_IRQ:                                        equ 0x01


; -------------------------------
; RX Status register
; -------------------------------
RX_STATUS_ADR:                                  equ 0x08
// There is no default value for this register.

; single flag bits & multi-bit-field masks
RX_ACK:                                         equ 0x80
RX_PKTERR:                                      equ 0x40
RX_EOPERR:                                      equ 0x20
RX_CRC0:                                        equ 0x10
RX_BAD_CRC:                                     equ 0x08
RX_DATCODE_LEN:                                 equ 0x04
RX_DATMODE_MSK:                                 equ 0x03


; -------------------------------
; RX Count register
; -------------------------------
RX_COUNT_ADR:                                   equ 0x09
RX_COUNT_RST:                                   equ 0x00
RX_COUNT_MSK:                                   equ 0xff


; -------------------------------
; RX Length Field register
; -------------------------------
RX_LENGTH_ADR:                                  equ 0x0a
RX_LENGTH_RST:                                  equ 0x00
RX_LENGTH_MSK:                                  equ 0xff


; -------------------------------
; Power Control register
; -------------------------------
PWR_CTRL_ADR:                                   equ 0x0b
PWR_CTRL_RST:                                   equ 0xa0

; single flag bits & multi-bit-field masks
PMU_EN:                                         equ 0x80
LV_IRQ_EN:                                      equ 0x40
PMU_SEN:                                        equ 0x20
PFET_OFF:                                       equ 0x10
LV_IRQ_TH_MSK:                                  equ 0x0c
PMU_OUTV_MSK:                                   equ 0x03

; LV_IRQ_TH values
LV_IRQ_TH_1P8_V:                                equ 0x0C
LV_IRQ_TH_2P0_V:                                equ 0x08
LV_IRQ_TH_2P2_V:                                equ 0x04
LV_IRQ_TH_PMU_OUTV:                             equ 0x00

; PMU_OUTV values
PMU_OUTV_2P4:                                   equ 0x03
PMU_OUTV_2P5:                                   equ 0x02
PMU_OUTV_2P6:                                   equ 0x01
PMU_OUTV_2P7:                                   equ 0x00


; -------------------------------
; Crystal Control register
; -------------------------------
XTAL_CTRL_ADR:                                  equ 0x0c
XTAL_CTRL_RST:                                  equ 0x04

; single flag bits & multi-bit-field masks
XOUT_FNC_MSK:                                   equ 0xc0
XS_IRQ_EN:                                      equ 0x20
XOUT_FREQ_MSK:                                  equ 0x07

; XOUT_FNC values
XOUT_FNC_XOUT_FREQ:                             equ 0x00
XOUT_FNC_PA_N:                                  equ 0x40
XOUT_FNC_RAD_STREAM:                            equ 0x80
XOUT_FNC_GPIO:                                  equ 0xC0

; XOUT_FREQ values
XOUT_FREQ_12MHZ:                                equ 0x00
XOUT_FREQ_6MHZ:                                 equ 0x01
XOUT_FREQ_3MHZ:                                 equ 0x02
XOUT_FREQ_1P5MHZ:                               equ 0x03
XOUT_FREQ_P75MHZ:                               equ 0x04


; -------------------------------
; I/O Configuration register
; -------------------------------
IO_CFG_ADR:                                     equ 0x0d
IO_CFG_RST:                                     equ 0x00
IO_CFG_MSK:                                     equ 0xff

; single flag bits & multi-bit-field masks
IRQ_OD:                                         equ 0x80
IRQ_POL:                                        equ 0x40
MISO_OD:                                        equ 0x20
XOUT_OD:                                        equ 0x10
PACTL_OD:                                       equ 0x08
PACTL_GPIO:                                     equ 0x04
SPI_3_PIN:                                      equ 0x02
IRQ_GPIO:                                       equ 0x01


; -------------------------------
; GPIO Control register
; -------------------------------
GPIO_CTRL_ADR:                                  equ 0x0e
GPIO_CTRL_RST:                                  equ 0x00
GPIO_CTRL_MSK:                                  equ 0xf0

; single flag bits & multi-bit-field masks
XOUT_OP:                                        equ 0x80
MISO_OP:                                        equ 0x40
PACTL_OP:                                       equ 0x20
IRQ_OP:                                         equ 0x10
XOUT_IP:                                        equ 0x08
MISO_IP:                                        equ 0x04
PACTL_IP:                                       equ 0x02
IRQ_IP:                                         equ 0x01


; -------------------------------
; Transaction Configuration register
; -------------------------------
XACT_CFG_ADR:                                   equ 0x0f
XACT_CFG_RST:                                   equ 0x80

; single flag bits & multi-bit-field masks
ACK_EN:                                         equ 0x80
FRC_END_STATE:                                  equ 0x20
END_STATE_MSK:                                  equ 0x1c
ACK_TO_MSK:                                     equ 0x03

; END_STATE field values
END_STATE_SLEEP:                                equ 0x00
END_STATE_IDLE:                                 equ 0x04
END_STATE_TXSYNTH:                              equ 0x08
END_STATE_RXSYNTH:                              equ 0x0C
END_STATE_RX:                                   equ 0x10

; ACK_TO field values
ACK_TO_4X:                                      equ 0x00
ACK_TO_8X:                                      equ 0x01
ACK_TO_12X:                                     equ 0x02
ACK_TO_15X:                                     equ 0x03


; -------------------------------
; Framing Configuration register
; -------------------------------
FRAMING_CFG_ADR:                                equ 0x10
FRAMING_CFG_RST:                                equ 0xa5

; single flag bits & multi-bit-field masks
SOP_EN:                                         equ 0x80
SOP_LEN:                                        equ 0x40
LEN_EN:                                         equ 0x20
SOP_THRESH_MSK:                                 equ 0x1f


; -------------------------------
; Data Threshold 32 register
; -------------------------------
DATA32_THOLD_ADR:                               equ 0x11
DAT32_THRESH_RST:                               equ 0x04
DAT32_THRESH_MSK:                               equ 0x0f


; -------------------------------
; Data Threshold 64 register
; -------------------------------
DATA64_THOLD_ADR:                               equ 0x12
DAT64_THRESH_RST:                               equ 0x0a
DAT64_THRESH_MSK:                               equ 0x1f


; -------------------------------
; RSSI register
; -------------------------------
RSSI_ADR:                                       equ 0x13
RSSI_RST:                                       equ 0x20

; single flag bits & multi-bit-field masks
SOP_RSSI:                                       equ 0x80
LNA_STATE:                                      equ 0x20
RSSI_LVL_MSK:                                   equ 0x1f


; -------------------------------
; EOP Control register
; -------------------------------
EOP_CTRL_ADR:                                   equ 0x14
EOP_CTRL_RST:                                   equ 0xa4

; single flag bits & multi-bit-field masks
HINT_EN:                                        equ 0x80
HINT_EOP_MSK:                                   equ 0x70
EOP_MSK:                                        equ 0x0f


; -------------------------------
; CRC Seed registers
; -------------------------------
CRC_SEED_LSB_ADR:                               equ 0x15
CRC_SEED_MSB_ADR:                               equ 0x16
CRC_SEED_LSB_RST:                               equ 0x00
CRC_SEED_MSB_RST:                               equ 0x00

; CRC related values
; USB CRC-16
CRC_POLY_MSB:                                   equ 0x80
CRC_POLY_LSB:                                   equ 0x05
CRC_RESI_MSB:                                   equ 0x80
CRC_RESI_LSB:                                   equ 0x0d


; -------------------------------
; TX CRC Calculated registers
; -------------------------------
TX_CRC_LSB_ADR:                                 equ 0x17
TX_CRC_MSB_ADR:                                 equ 0x18


; -------------------------------
; RX CRC Field registers
; -------------------------------
RX_CRC_LSB_ADR:                                 equ 0x19
RX_CRC_MSB_ADR:                                 equ 0x1a
RX_CRC_LSB_RST:                                 equ 0xff
RX_CRC_MSB_RST:                                 equ 0xff


; -------------------------------
; Synth Offset registers
; -------------------------------
TX_OFFSET_LSB_ADR:                              equ 0x1b
TX_OFFSET_MSB_ADR:                              equ 0x1c
TX_OFFSET_LSB_RST:                              equ 0x00
TX_OFFSET_MSB_RST:                              equ 0x00

; single flag bits & multi-bit-field masks
STRIM_MSB_MSK:                                  equ 0x0f
STRIM_LSB_MSK:                                  equ 0xff


; -------------------------------
; Mode Override register
; -------------------------------
MODE_OVERRIDE_ADR:                              equ 0x1d
MODE_OVERRIDE_RST:                              equ 0x00

FRC_AWAKE:                                      equ 0x03
FRC_AWAKE_OFF_1:                                equ 0x01
FRC_AWAKE_OFF_2:                                equ 0x00

; single flag bits & multi-bit-field masks
DIS_AUTO_SEN:                                   equ 0x80
SEN_TXRXB:                                      equ 0x40
FRC_SEN:                                        equ 0x20
FRC_AWAKE_MSK:                                  equ 0x18
MODE_OVRD_FRC_AWAKE:                            equ 0x18
MODE_OVRD_FRC_AWAKE_OFF_1:                      equ 0x08
MODE_OVRD_FRC_AWAKE_OFF_2:                      equ 0x00
RST:                                            equ 0x01
FRC_PA:                                         equ 0x02


; -------------------------------
; RX Override register
; -------------------------------
RX_OVERRIDE_ADR:                                equ 0x1e
RX_OVERRIDE_RST:                                equ 0x00

; single flag bits & multi-bit-field masks
ACK_RX:                                         equ 0x80
EXTEND_RX_TX:                                   equ 0x40
MAN_RXACK:                                      equ 0x20
FRC_RXDR:                                       equ 0x10
DIS_CRC0:                                       equ 0x08
DIS_RXCRC:                                      equ 0x04
ACE:                                            equ 0x02


; -------------------------------
; TX Override register
; -------------------------------
TX_OVERRIDE_ADR:                                equ 0x1f
TX_OVERRIDE_RST:                                equ 0x00

; single flag bits & multi-bit-field masks
ACK_TX_SEN:                                     equ 0x80
FRC_PREAMBLE:                                   equ 0x40
DIS_TX_RETRANS:                                 equ 0x20
MAN_TXACK:                                      equ 0x10
OVRRD_ACK:                                      equ 0x08
DIS_TXCRC:                                      equ 0x04
CO:                                             equ 0x02
TXINV:                                          equ 0x01


;------------------------------------------------------------------------------
;      File Function Detail
;------------------------------------------------------------------------------

; -------------------------------
; TX Buffer - 16 bytes
; -------------------------------
TX_BUFFER_ADR:                                  equ 0x20


; -------------------------------
; RX Buffer - 16 bytes
; -------------------------------
RX_BUFFER_ADR:                                  equ 0x21


; -------------------------------
; Framing Code - 8 bytes
; -------------------------------
SOP_CODE_ADR:                                   equ 0x22

; CODESTORE_REG_SOF_RST        64'h17_ff_9e_21_36_90_c7_82
CODESTORE_BYTE7_SOF_RST:                        equ 0x17
CODESTORE_BYTE6_SOF_RST:                        equ 0xff
CODESTORE_BYTE5_SOF_RST:                        equ 0x9e
CODESTORE_BYTE4_SOF_RST:                        equ 0x21
CODESTORE_BYTE3_SOF_RST:                        equ 0x36
CODESTORE_BYTE2_SOF_RST:                        equ 0x90
CODESTORE_BYTE1_SOF_RST:                        equ 0xc7
CODESTORE_BYTE0_SOF_RST:                        equ 0x82


; -------------------------------
; Data Code - 16 bytes
; -------------------------------
DATA_CODE_ADR:                                  equ 0x23

; CODESTORE_REG_DCODE0_RST            64'h01_2B_F1_DB_01_32_BE_6F
CODESTORE_BYTE7_DCODE0_RST:                     equ 0x01
CODESTORE_BYTE6_DCODE0_RST:                     equ 0x2b
CODESTORE_BYTE5_DCODE0_RST:                     equ 0xf1
CODESTORE_BYTE4_DCODE0_RST:                     equ 0xdb
CODESTORE_BYTE3_DCODE0_RST:                     equ 0x01
CODESTORE_BYTE2_DCODE0_RST:                     equ 0x32
CODESTORE_BYTE1_DCODE0_RST:                     equ 0xbe
CODESTORE_BYTE0_DCODE0_RST:                     equ 0x6f

; CODESTORE_REG_DCODE1_RST            64'h02_F9_93_97_02_FA_5C_E3
CODESTORE_BYTE7_DCODE1_RST:                     equ 0x02
CODESTORE_BYTE6_DCODE1_RST:                     equ 0xf9
CODESTORE_BYTE5_DCODE1_RST:                     equ 0x93
CODESTORE_BYTE4_DCODE1_RST:                     equ 0x97
CODESTORE_BYTE3_DCODE1_RST:                     equ 0x02
CODESTORE_BYTE2_DCODE1_RST:                     equ 0xfa
CODESTORE_BYTE1_DCODE1_RST:                     equ 0x5c
CODESTORE_BYTE0_DCODE1_RST:                     equ 0xe3


; -------------------------------
; Preamble - 3 bytes
; -------------------------------
PREAMBLE_ADR:                                   equ 0x24

PREAMBLE_CODE_MSB_RST:                          equ 0x33
PREAMBLE_CODE_LSB_RST:                          equ 0x33
PREAMBLE_LEN_RST:                               equ 0x02


; -------------------------------
; Laser Fuses - 8 bytes (2 hidden)
; -------------------------------
MFG_ID_ADR:                                     equ 0x25


; -------------------------------
; XTAL Startup Delay
; -------------------------------
XTAL_CFG_ADR:                                   equ 0x26
XTAL_CFG_RST:                                   equ 0x00

; -------------------------------
; Clock Override
; -------------------------------
CLK_OVERRIDE_ADR:                               equ 0x27
CLK_OVERRIDE_RST:                               equ 0x00

RXF:                                            equ 0x02


; -------------------------------
; Clock Enable
; -------------------------------
CLK_EN_ADR:                                     equ 0x28
CLK_EN_RST:                                     equ 0x00

RXF:                                            equ 0x02


; -------------------------------
; Receiver Abort
; -------------------------------
RX_ABORT_ADR:                                   equ 0x29
RX_ABORT_RST:                                   equ 0x00

ABORT_EN:                                       equ 0x20


; -------------------------------
; Auto Calibration Time
; -------------------------------
AUTO_CAL_TIME_ADR:                              equ 0x32
AUTO_CAL_TIME_RST:                              equ 0x0C

AUTO_CAL_TIME_MAX:                              equ 0x3C


; -------------------------------
; Auto Calibration Offset
; -------------------------------
AUTO_CAL_OFFSET_ADR:                            equ 0x35
AUTO_CAL_OFFSET_RST:                            equ 0x00

AUTO_CAL_OFFSET_MINUS_4:                        equ 0x14

