;-----------------------------------------------------------------------------
;
;  Filename:    lpSpi.asm
;
;  Description: SPI based LP Radio access functions
;
;           PSoC or EnCoRe2/PRoC SPI is supported via an external EQU
;
;                    3 or 4 wire SPI is supported via an external EQU
;
;               Multiplexed MOSI/IRQ is supported via an external EQU
;
;-----------------------------------------------------------------------------
; WirelessUSB LP Radio Driver Version 1.2
;-----------------------------------------------------------------------------
;
; Copyright 2005-2006, Cypress Semiconductor Corporation.
;
; This software is owned by Cypress Semiconductor Corporation (Cypress)
; and is protected by and subject to worldwide patent protection (United
; States and foreign), United States copyright laws and international
; treaty provisions. Cypress hereby grants to licensee a personal,
; non-exclusive, non-transferable license to copy, use, modify, create
; derivative works of, and compile the Cypress Source Code and derivative
; works for the sole purpose of creating custom software in support of
; licensee product to be used only in conjunction with a Cypress integrated
; circuit as specified in the applicable agreement. Any reproduction,
; modification, translation, compilation, or representation of this
; software except as specified above is prohibited without the express
; written permission of Cypress.
;
; Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
; WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
; Cypress reserves the right to make changes without further notice to the
; materials described herein. Cypress does not assume any liability arising
; out of the application or use of any product or circuit described herein.
; Cypress does not authorize its products for use as critical components in
; life-support systems where a malfunction or failure may reasonably be
; expected to result in significant injury to the user. The inclusion of
; Cypress' product in a life-support systems application implies that the
; manufacturer assumes all risk of such use and in doing so indemnifies
; Cypress against all charges.
;
; Use may be limited by and subject to the applicable Cypress software
; license agreement.
;
;-----------------------------------------------------------------------------

;--------------------------------------------------------------------------;
;                                                                          ;
;                      I N C L U D E   F I L E S                           ;
;                                                                          ;
;--------------------------------------------------------------------------;

        INCLUDE "lpradio.inc"
        INCLUDE "spim_radio.inc"
        INCLUDE "psocgpioint.inc"
        INCLUDE "lpregs.inc"
        INCLUDE "lpIrqMacros.inc"
        INCLUDE "memory.inc"

;-----------------------------------------------------------------------------
  IF (LP_SPI)           ; Just shorter names for SPI I/O for local scope
SPI_CTRL_REG:   EQU     SPIM_Radio_CONTROL_REG
SPI_TX_REG:     EQU     SPIM_Radio_TX_BUFFER_REG
SPI_RX_REG:     EQU     SPIM_Radio_RX_BUFFER_REG
SPI_TX_EMPTY:   EQU     SPIM_Radio_SPIM_TX_BUFFER_EMPTY
SPI_DONE:       EQU     SPIM_Radio_SPIM_SPI_COMPLETE
SPI_RX_FULL:    EQU     SPIM_Radio_SPIM_RX_BUFFER_FULL
  ELSE ; not LP_SPI
SPI_RX_REG:     EQU     SPIDATA         ; PRoC/enCoRe II use same reg
SPI_TX_REG:     EQU     SPIDATA         ; PRoC/enCoRe II use same reg
  ENDIF


;--------------------------------------------------------------------------;
;                                                                          ;
;                            V A R I A B L E S                             ;
;                                                                          ;
;--------------------------------------------------------------------------;
        AREA    InterruptRAM(ram)       ; Variables on RAM Page 0

  IF SYSTEM_LARGE_MEMORY_MODEL
        ; Although contrary to the big-endian ImageCraft compiler, 
        ;  Little-endian makes cleaner LMM/SMM macros.
RadioWipPtr::   BLK     2       ; Working pointer (LSB, MSB, little-endian)
RadioWipLen::   BLK     1       ; Working buffer length.
RadioPtr::      BLK     2       ; SPI access routine parameter (little-endian)
RadioLen::      BLK     1       ; SPI access routines parameter - length.
  ELSE
RadioWipPtr::   BLK     1       ; Working pointer.
RadioWipLen::   BLK     1       ; Working buffer length.
RadioPtr::      BLK     1       ; Parameter to SPI access routines - address.
RadioLen::      BLK     1       ; Parameter to SPI access routines - length.
  ENDIF

RadioSpiTemp:   BLK     1       ; Local working temp.

;--------------------------------------------------------------------------;
;                                                                          ;
;                  M A C R O   D E F I N I T I O N S                       ;
;                                                                          ;
;--------------------------------------------------------------------------;
macro RADIO_SELECT
  IF PORT_SHADOW_ENABLE    ; Avoid side-effects on other bits of LP_nSS PORT
        push    a
        and        [Port_0_Data_SHADE + LP_nSS_Port], ~LP_nSS_MASK
        mov     a, [Port_0_Data_SHADE + LP_nSS_Port]
        mov     reg[LP_nSS_Data_ADDR], a                ; Assert Slave Select
        pop     a
  ELSE
        and     reg[LP_nSS_Data_ADDR],~LP_nSS_MASK      ; Assert Slave Select
  ENDIF
; ------------------------------
  IF MUXED_IRQ
        or      reg[P15CR], (SPI_USE | OUTPUT_EN)       ; P1.5 is SPI MOSI out
  ENDIF
endm

macro RADIO_DESELECT
  IF MUXED_IRQ
        and     reg[P15CR], ~(SPI_USE | OUTPUT_EN)      ; P1.5 is LP IRQ input
  ENDIF
; ------------------------------
  IF PORT_SHADOW_ENABLE    ; Avoid side-effects on other bits of LP_nSS PORT
        push    a
        or         [Port_0_Data_SHADE + LP_nSS_Port], LP_nSS_MASK
        mov     a, [Port_0_Data_SHADE + LP_nSS_Port]
        mov     reg[LP_nSS_Data_ADDR], a                ; Release Slave Select
        pop     a
  ELSE
        or      reg[LP_nSS_Data_ADDR], LP_nSS_MASK      ; Release Slave Select
  ENDIF
endm
;-----------------------------------------------------------------------------
macro E2_CLR_SPI_RXTX_FLAGS
  IF (LP_SPI)           ; PSoC SPI does nothing
  ELSE ; PRoC/enCoRe II ------------------------------------------------------
        mov     reg[INT_CLR0],~(INT_MSK0_SPI_TX | INT_MSK0_SPI_RX)
  ENDIF
endm
;-----------------------------------------------------------------------------
SPI_USE:        equ     0x80
OUTPUT_EN:      equ     0x01
SPI_SWAP:       equ     0x80
;-----------------------------------------------------------------------------
macro   SET_SPI_IN
  IF THREE_WIRE_SPI
        or      reg[SPICR], SPI_SWAP    ; Make the I/O pin an input
  ENDIF ;THREE_WIRE_SPI
endm
;-----------------------------------------------------------------------------
macro   SET_SPI_OUT
  IF THREE_WIRE_SPI
        and     reg[SPICR], ~SPI_SWAP   ; Make the I/O pin an output
  ENDIF ;THREE_WIRE_SPI
endm
;-----------------------------------------------------------------------------
macro   SPI_TX_EMPTY_WAIT
  IF (LP_SPI)   ; ---------------------------------------------------------
.L2:    TST     REG[SPI_CTRL_REG], SPI_TX_EMPTY ; (TST opcode preserves CRY)
  ELSE ; PRoC/enCoRe II ------------------------------------------------------
.L2:    TST     REG[INT_CLR0], INT_MSK0_SPI_TX  ; (TST opcode preserves CRY)
  ENDIF                 ; ----------------------------------------------------
        JZ      .L2
endm
macro   SPI_RX_EMPTY_WAIT
  IF (LP_SPI)   ; ---------------------------------------------------------
.L2:    TST     REG[SPI_CTRL_REG], SPI_TX_EMPTY ; (TST opcode preserves CRY)
  ELSE ; PRoC/enCoRe II ------------------------------------------------------
.L2:    TST     REG[INT_CLR0], INT_MSK0_SPI_RX  ; (TST opcode preserves CRY)
  ENDIF                 ; ----------------------------------------------------
        JZ      .L2
endm
;-----------------------------------------------------------------------------

;--------------------------------------------------------------------------;
;                                                                          ;
;                                 C O D E                                  ;
;                                                                          ;
;--------------------------------------------------------------------------;

        AREA    UserModules (ROM, REL)

        DISABLE_CODE_COMPRESSION

;-----------------------------------------------------------------------------
;
; Some comments on the operation of the SPI PSoC block:
;
; The SPI_DONE and SPI_TX_EMPTY status
;  bits are self-clearing. After you've read SPI_CTRL_REG these
;  bits will be cleared. During a multibyte SPI transaction we use the 
;  SPI_TX_EMPTY status to pace the data, but at the end
;  of a transaction we use SPI_DONE to determine that
;  we can de-assert the slave select.
;
; Although the hardware generates the SPI_CTRL_REG status
;  one full byte time before SPI_TX_EMPTY if you are
;  not reading the SPI_CTRL_REG register fast enough you might
;  see both of those status indications occur together, and after you've
;  read the SPI_CTRL_REG status it will clear and you will not see
;  it again. In the case that the SPI clock is very fast, or if interrupts 
;  are left enabled during the execution of these routines then it is very
;  possible to see both of these indications occur together.
;
; In order for these routines to allow interrupts to be enabled during their
;  execution they are structured to allow the assertion of the two status
;  bits SPI_DONE and SPI_TX_EMPTY
;  on the same read of SPI_CTRL_REG. Although this is a real world
;  condition, you will only see it under stressed conditions.
;
; One other strangeness of the PSoC SPI block is that the
;  SPI_DONE status ONLY INDICATES THE STATE OF THE
;  SHIFTER - it does not include the state of the holding register. This
;  means that when we need to poll SPI_DONE at the
;  end of a transaction we need to make sure we see the status incdication
;  SPI_TX_EMPTY to know that the complete wasn't for
;  the previous byte.
;
;-----------------------------------------------------------------------------

;--------------------------------------------------------------------------;
;                                                                          ;
;                      I N I T I A L I Z A T I O N                         ;
;                                                                          ;
;--------------------------------------------------------------------------;

.section
;-----------------------------------------------------------------------------
;
; RadioReset: Send Soft Reset command to radio.
;
; 'C' Call:        void RadioReset(void);
;
; Assembly Call:   A: none
;                  X: none
;
; Assembly Return: A: lost
;                  X: lost
;             CUR_PP: Large Memory Model sets to RadioDriverRamPage
;-----------------------------------------------------------------------------
_RadioReset::
 RadioReset::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        RADIO_DESELECT

        MOV     A, MODE_OVERRIDE_ADR    ; Reset Radio through SPI (soft reset)
        MOV     X, RST                  ;  (don't use LP RST pin)

  IF (MUXED_IRQ & THREE_WIRE_SPI)
        CALL    RadioWrite              ; Issue soft-reset                 
        MOV     A, IO_CFG_ADR           ;    
        MOV     X, SPI_3_PIN | IRQ_GPIO ; Enable IRQ/MOSI mux + 3-wire SPI
  ELSE
    IF MUXED_IRQ; and non-3-wire SPI
        CALL    RadioWrite              ; Issue soft-reset                 
        MOV     A, IO_CFG_ADR           ;    
        MOV     X, IRQ_GPIO             ; Only enable IRQ muxed on MOSI pin
    ENDIF
    IF THREE_WIRE_SPI; and non-Muxed IRQ
        CALL    RadioWrite              ; Issue soft-reset                 
        MOV     A, IO_CFG_ADR           ;    
        MOV     X, SPI_3_PIN            ; Only enable 3-wire SPI
    ENDIF
  ENDIF
        JMP     RadioWrite
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioWriteSwapped: Write a single byte to Radio register. 
;                  Same as RadioWrite(), except A and X are swapped because
;                  C calling convention makes it easier to set registers.
;
; Assembly Call:   A: Value to write to selected register.
;                  X: Register number to write, high 2 bits MUST be ZERO !
; -------------------
;
; RadioWrite:      Write a single byte to an Radio register.
;
; 'C' Call:        void RadioWrite(RADIO_REG_ADDR regAddr, BYTE value);
;
; Assembly Call:   A: Register number to write, high 2 bits MUST be ZERO !
;                  X: Value to write to the selected register.
;
; Assembly Return: A: lost
;                  X: lost
;             CUR_PP: Large Memory Model sets to RadioDriverRamPage
;-----------------------------------------------------------------------------
 RadioWriteSwapped::
        SWAP    A, X

 RadioWrite::
_RadioWrite::
        OR      A, bSPI_WRITE           ; Set the Write Bit.

  IF THREE_WIRE_SPI   ; ------------------------------------------------------
        ; Note: 3-wire SPI Write can't fallthru to use Read routine
        RAM_SETPAGE_CUR >RadioDriverRamPage
        PUSH_F_VIA_RAM
        RADIO_SELECT

        E2_CLR_SPI_RXTX_FLAGS           ; Clear TX and RX flag
        MOV     REG[SPI_TX_REG], A      ; Write SPI adr

.L1:    TST     REG[INT_CLR0],INT_MSK0_SPI_RX ; Address transfer complete?
        JZ      .L1

        E2_CLR_SPI_RXTX_FLAGS           ; Clear the TX and RX flag
        MOV     A, X                    ; Data from address read is garbage
        MOV     REG[SPI_TX_REG], A      ; Write SPI data

.L2:    TST     REG[INT_CLR0],INT_MSK0_SPI_RX ; Data transfer complete?
        JZ      .L2

        RADIO_DESELECT
        POP_F_RET                       ; 3-wire SPI returns here
  ENDIF ; THREE_WIRE_SPI    ; ------------------------------------------------
  
        ; 4-wire SPI falls-thru to use RadioRead() to perform the Write
             
;-----------------------------------------------------------------------------
;
; RadioRead:       Read a single byte from an Radio register.
;
; 'C' Call:        BYTE RadioRead(RADIO_REG_ADDR regAddr);
;
; Assembly Call:   A: The register number to read
;                  X: READ=unused, fallthru from WRITE=Data
;
;                  (For both the 'C' and assembly call the top two bits of
;                   the register number MUST be clear.)
;
; Assembly Return: A: Value from register
;                  X: lost
;             CUR_PP: Large Memory Model sets to RadioDriverRamPage
;-----------------------------------------------------------------------------
 RadioRead::
_RadioRead::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        PUSH_F_VIA_RAM
        RADIO_SELECT

        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write SPI address
        SPI_RX_EMPTY_WAIT

        SET_SPI_IN                      ; 3-wire read flips MOSI to MISO
        E2_CLR_SPI_RXTX_FLAGS
        MOV     A, X
        MOV     REG[SPI_TX_REG], A      ; Write SPI data
        SPI_RX_EMPTY_WAIT

  IF (LP_SPI)   ; ------------------------------------------------------------
.L3:    TST     REG[SPI_CTRL_REG], SPI_DONE
        JZ      .L3
ENDIF

        E2_CLR_SPI_RXTX_FLAGS
        MOV     A, REG[SPI_RX_REG]      ; Get SPI Rx data
  
        SET_SPI_OUT                     ; 3-wire idles as MOSI
                                        ; A = return value
        RADIO_DESELECT
        POP_F_RET                       ; Restore F and return
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioReadRxStatusDebounced:
;                  Read a single byte from the RX_IRQ_STATUS_ADR register and
;                  debounce the update of the RXC and RXE bits. If only one of
;                  those two bits is set, read the register a second time and
;                  or them together. This second read happens in the same
;                  SPI transaction as a burst to the same address.
;
; 'C' Call:        BYTE RadioReadDebounced(RADIO_REG_ADDR regAddr);
;
; Assembly Call:   A: Register # to read
;                  X: none
;
; Assembly Return: A: Value from register
;                  X: Untouched
;             CUR_PP: Large Memory Model sets to RadioDriverRamPage
;-----------------------------------------------------------------------------
 RadioReadStatusDebounced::
_RadioReadStatusDebounced::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        PUSH_F_VIA_RAM
        RADIO_SELECT

        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write SPI address.
        SPI_RX_EMPTY_WAIT

        SET_SPI_IN                   
        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write SPI data.
        SPI_RX_EMPTY_WAIT

  IF (LP_SPI)   ; ---------------------------------------------------------
.L3:    TST     REG[SPI_CTRL_REG], SPI_DONE
        JZ      .L3
  ENDIF

        ; If RXC and RXE (or TXC and TXE) bits match, 
        ; then skip the debounce read
        MOV     A, REG[SPI_RX_REG]      ; Get SPI Rx data
        MOV     [RadioSpiTemp], A
        AND     A, RXC_IRQ | RXE_IRQ
        JZ      .Done                   ; Both bits are ZERO
        CMP     A, RXC_IRQ | RXE_IRQ
        JZ      .Done                   ; Both bits are ONE

        ; The Complete and Error bits are different. 
        ; Read again to ensure both bits have updated.
        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A             
        SPI_RX_EMPTY_WAIT
  IF (LP_SPI)   ; ---------------------------------------------------------
.L5:    TST     REG[SPI_CTRL_REG],SPI_DONE
        JZ      .L5
  ENDIF ;       ; ---------------------------------------------------------
        MOV     A, REG[SPI_RX_REG]      ; Get SPI Rx data
        OR      [RadioSpiTemp], A

.Done:  MOV     A, [RadioSpiTemp]       ; Get the return value.
  
        SET_SPI_OUT

        RADIO_DESELECT
        POP_F_RET
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioSetPtr:     Set the buffer pointer address for RadioBurstRead,
;                  RadioFileRead, RadioBurstWrite and RadioFileWrite functions
;
; 'C' Call:        void RadioSetPtr(unsigned char ramPtr);
;
; Return:     CUR_PP: Large Memory Model sets to RadioDriverRamPage
;-----------------------------------------------------------------------------
_RadioSetPtr::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioPtr+0], X         ; LSByte of pointer
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioPtr+1], A         ; LMM: MSB 16-bit pointer
  ENDIF
        RET

.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioSetLength:  Set the buffer length for RadioBurstRead & RadioFileRead.
;
; 'C' Call:        void RadioSetLength(unsigned char length);
;
; Return:     CUR_PP: Large Memory Model sets to RadioDriverRamPage
;-----------------------------------------------------------------------------
_RadioSetLength::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioLen], A
        RET

.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioBurstWriteWip:
;                  Write sequence of bytes to sequence of Radio registers.
;                  Uses RadioWipPtr instead of RadioPtr as the data pointer.                   
;
; ---------------------
;
; RadioFileWriteWip:
;                  Write sequence of bytes to singleRadio register.
;                  Uses RadioWipPtr instead of RadioPtr as the data pointer.                   
;
; ---------------------
;
; RadioBurstWrite: Write sequence of bytes to sequence of Radio registers.
;
; 'C' Call:        void RadioBurstWrite(BYTE regAddr, BYTE cnt);
;                  Must set RadioPtr via RadioSetPtr() before RadioBurstWrite
;                  
; ---------------------
;
; RadioFileWrite:  Write sequence of bytes to single Radio register.
;
; 'C' Call:        void RadioFileWrite(LS_REG_ADDR regAddr, BYTE cnt);
;                  (Must set RadioPtr via RadioSetPtr() before RadioFileWrite)
; ---------------------
;
; Assembly Call:   A: Register number to write, top two bits MUST be clear!
;                  X: Length of buffer.
;           RadioPtr: Address of buffer to write.
;
; Assembly Return: A: Undefined
;                  X: Undefined
;             CUR_PP: Large Memory Model sets to RadioDriverRamPage
;             MVR_PP: Large Memory Model sets to RadioWipPtr page
;-----------------------------------------------------------------------------
 RadioBurstWriteWip::
        OR      A, bSPI_AUTO_INC        ; Set increment flag
        JMP     RadioFileWriteWip       ; Use existing WIP ptr/ctr

 RadioBurstWrite::
_RadioBurstWrite::
        OR      A, bSPI_AUTO_INC        ; Set increment flag.

 RadioFileWrite::
_RadioFileWrite::
        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], [RadioPtr+1] ; Copy MSByte buffer pointer.
  ENDIF
        MOV     [RadioWipPtr+0], [RadioPtr+0] ; Copy LSByte buffer pointer.

RadioFileWriteWip::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        PUSH_F_VIA_RAM  
        DEC     X                       ; If the length is zero
        JC      .Done                   ;  do nothing at all.
        OR      A, bSPI_WRITE           ; Set the write flag. 
        RADIO_SELECT

        E2_CLR_SPI_RXTX_FLAGS           
        MOV     REG[SPI_TX_REG], A      ; Write the address

  IF (LP_SPI)   ; ---------------------------------------------------------
    IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     A, [RadioWipPtr+1]      ; MSByte adr 
        RAM_SETPAGE_MVR A               ; MVR operates on RadioWipPtr page
    ENDIF       
.Burst: MVI     A, [RadioWipPtr+0]      ; Get the data
        SPI_TX_EMPTY_WAIT
        MOV     REG[SPI_TX_REG], A
        DEC     X
        JNC     .Burst

        ; All done bursting, wait for last byte to leave the holding register
        SPI_TX_EMPTY_WAIT
                    
        ; Wait for shifting to complete and the SPI transaction to finish.
.L1:    TST     REG[SPI_CTRL_REG],SPI_DONE
        JZ      .L1

  ELSE ; PRoC/enCoRe II ------------------------------------------------------

    IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     A, [RadioWipPtr+1]      ; MSByte adr 
        RAM_SETPAGE_MVR A               ; MVR operates on RadioWipPtr page
    ENDIF       
.Burst: 
        MVI     A, [RadioWipPtr+0]      ; Get the data.
        SPI_TX_EMPTY_WAIT
        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write the address
        DEC     X
        JNC     .Burst

        ; All done bursting, wait for last byte to leave the holding register
        SPI_TX_EMPTY_WAIT

        ; Clear RX flag, for next Loop, it takes 9+4+9=22 cycles to finish
        MOV     REG[INT_CLR0], ~INT_MSK0_SPI_RX 
                    
        ; Wait for shifting to complete and the SPI transaction to finish.
.L1:    TST     REG[INT_CLR0],INT_MSK0_SPI_RX
        JZ      .L1
        E2_CLR_SPI_RXTX_FLAGS
  ENDIF ; LP_SPI        ; ----------------------------------------------------
        RADIO_DESELECT
.Done:
        POP_F_RET

.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioBurstReadWip:
;                  Read a sequence of bytes from a sequence of Radio registers
;                  using RadioWipPtr as the buffer pointer.
;
; Assembly Call:   RadioWipPtr: Address of buffer to read.
;                  A: The register number to write.
;                     (Top two bits MUST be clear.)
;                  X: Length of the buffer.
;
; Assembly Return: A: Undefined
;                  X: Undefined
;-----------------------------------------------------------------------------
 RadioBurstReadWip::
        OR      A, bSPI_AUTO_INC        ; Set the increment flag.
        JMP     RadioFileReadWip

.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioBurstRead:  Read a sequence of bytes from a sequence of Radio registers
;
; 'C' Call:        void RadioBurstRead(unsigned char cnt);
;                  Must set RadioPtr via RadioSetPtr() before RadioBurstRead.
;                  
; See RadioFileRead() for entry/exit parameters
;-----------------------------------------------------------------------------
 RadioBurstRead::
_RadioBurstRead::
        OR      A,bSPI_AUTO_INC         ; Set the increment flag.

;-----------------------------------------------------------------------------
;
; RadioFileRead:   Read a sequence of bytes from a single Radio registers.
;
;                      time->           |8spiclk|8spiclk|8spiclk|
;                  Assert Slave Select  |       |       |       |
;                  Write Address        |       |       |       |
;                  Wait for Empty(1)----+       |       |       |
;                  Write Nul Data               |       |       |
;                  Wait for Empty(2)------------+       |       |
;                  Write Nul Data                       |       |
;                  Read Nul Data                        |       |
;                  Wait for Empty(3)--------------------+       |
;                  Write Nul Data                               |
;                  Read Data                                    |
;                  Wait for Empty and complete------------------+
;                  Read Data
;                  De-assert Slave Select
;
;                  The main loop of this routine is a littler slower than it 
;                  should be. With a 12MHz CPU the burst is not always 
;                  continuous. There are occasional gaps between bytes in the
;                  burst. It should be possible to correct this, but it's a
;                  minor problem and this is already pretty convoluted to deal
;                  with the latency caused by the TX and RX holding buffers.
;
;                  Properly handles all length cases, including zero and one. 
;
; 'C' Call:        (Must set RadioPtr and RadioLen by calling RadioSetPtr 
;                  prior to RadioFileRead.)
;                  void RadioFileRead(BYTE regAddr, BYTE cnt);
;
; Assembly Call:   A: The register number to write, top 2 bits MUST be clear!
;                  X: Length of the buffer.
;           RadioPtr: Address of buffer to read.
;           RadioLen: Size of buffer
;
; Assembly Return: A: Undefined
;                  X: Undefined
;             CUR_PP: Large Memory Model sets to RadioDriverRamPage
;             STK_PP: Large Memory Model sets to SYSTEM_STACK_PAGE
;-----------------------------------------------------------------------------
 RadioFileRead::
_RadioFileRead::
        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], [RadioPtr+1] ; LMM: MSB working pointer
  ENDIF
        MOV     [RadioWipPtr+0], [RadioPtr+0] ; Initialize working pointer
        MOV     [RadioWipLen], [RadioLen]     ;  and length.

RadioFileReadWip::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        PUSH_F_VIA_RAM                  ; PUSH F and disable GIE
        DEC     X                       ; Check for zero length case.
        JC      .exit0                  ; If zero, nothing to do.

        ; Here, X = number of bytes in the file read -1. The final byte is
        ;  handled specially by waiting for COMPLETE instead of EMPTY at
        ;  .LastByte below.
        ; --------------------------------------------------------------------
        RADIO_SELECT
        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write SPI address.

        ; --------------------------------------------------------------------
        ; Using the SP as a write data pointer is 6-cycles faster than MVI.
        ;       (Since GIE is already disabled, using SP is safe).
        ; --------------------------------------------------------------------
        SWAP    A, SP
        SWAP    [RadioWipPtr+0], A      ; LSByte of pointer
        SWAP    A, SP                   ; SP = RadioWipPtr
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     A, [RadioWipPtr+1]      ; MSByte of pointer (page #)
        MOV     REG[STK_PP], A
  ENDIF

        SPI_RX_EMPTY_WAIT                   
        SET_SPI_IN                   
        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write null data.
;
; We need to get three items set up for this execution of the routine and the
; next as well based on whether there is room in the buffer.
;                bytes to read(X) - space in buffer(RadioWipLen)
;                C (RadioWipLen>X)  NC (RadioWipLen<=X)
;-----------------------------------------------------------------------------
; Reads that fit in buffer : X             = X                = RadioWipLen
; Reads thrown away        : RadioSpiTemp  = 0                = X-RadioWipLen
; Bytes left in buffer     : RadioWipLen   = RadioWipLen-X    = 0
;                                                         
        MOV     A, X
        SUB     A, [RadioWipLen]        ; CRY = 1 if user's buffer TOO SMALL
        SPI_TX_EMPTY_WAIT               ; cry preserved
        JNC     .S1                     ; User buf   holds ALL   Radio Rx data

        MOV     [RadioSpiTemp], 0       ; User buf too small for Radio Rx data
        MOV     A, X
        SUB     [RadioWipLen], A
        JMP     .BurstBottom

        ; A = number of bytes we are going to move -1.
        ; We need to subtract the total number of bytes which is A+1.
        ;
.S1:    MOV     [RadioSpiTemp], A
        MOV     X, [RadioWipLen]
        MOV     [RadioWipLen], 0
        JMP     .BurstBottom

.BurstLoop1:
        E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A      ; Write null data.
        SPI_TX_EMPTY_WAIT
        MOV     A, REG[SPI_RX_REG]      ; Get SPI Rx data
        PUSH    A                       ; Save data in buffer, not real stack
                        
.BurstBottom:
        DEC     X                       ; Count
        JNC     .BurstLoop1             ; Keep copying to User Buffer

        ; User Buffer is FULL, if Radio still has Rx data then read it and 
        ; discard.  If more data to dump, count is in RadioSpiTemp.
        MOV     X, [RadioSpiTemp]
        JMP     .trash0

.trash: E2_CLR_SPI_RXTX_FLAGS
        MOV     REG[SPI_TX_REG], A        
        SPI_TX_EMPTY_WAIT               ; Don't bother reading SPI Rx data
.trash0:                                ; because User's Buffer is FULL
        DEC     X
        JNC     .trash              

        ; --------------------------------------------------------------------
        ; Moved all data except for one last byte. Wait for it and if 
        ;  we have room in the buffer, get it and store it.
        ; --------------------------------------------------------------------
.LastByte:
  IF (LP_SPI)   ; ---------------------------------------------------------
        TST     REG[SPI_CTRL_REG], SPI_DONE
  ELSE ; PRoC/enCoRe II ------------------------------------------------------
        TST     REG[INT_CLR0], INT_MSK0_SPI_RX 
  ENDIF                 ; ----------------------------------------------------
        JZ      .LastByte
        CMP     [RadioWipLen], 0
        JZ      .exit1                  ; Exit w/cleanup
                   
        MOV     A, REG[SPI_RX_REG]      ; Get SPI Rx data 
        PUSH    A                       ; Save data in buffer, not real stack
        DEC     [RadioWipLen]
                        
.exit1:
        E2_CLR_SPI_RXTX_FLAGS
        SET_SPI_OUT
        RADIO_DESELECT
        SWAP    A, SP                   ; Restore SP to STACK
        SWAP    [RadioWipPtr], A        ; Restore SP to STACK
        SWAP    A, SP                   ; Restore SP to STACK
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     REG[STK_PP], SYSTEM_STACK_PAGE
  ENDIF
                   
.exit0:
        POP_F_RET                       ; Restore GIE bit
.endsection

        ENABLE_CODE_COMPRESSION

; ############################################################################
; END OF lpSpi.asm
; ############################################################################
