//-----------------------------------------------------------------------------
// PSoC RF Remote LED-Light Demo RX 
// 
// PSoCEval1 Connections:
//      P1.2    : Digital Input,  IRQ
//      P1.3    : Digital Output, nSS
//      P1.4    : Digital Output, SCLK
//      P1.5    : Digital Output, MOSI
//      P1.6    : Digital Input,  MISO
//      P1.7    : Digital Output, RST
// Build by Imagecraft C-comiler, PSoC Designer Version 4.4 with SP2
//-----------------------------------------------------------------------------

#include <m8c.h>                       
#include "PSoCAPI.h"
#include "PSoCgpioint.h"
#include "lpregs.h"
#include "lpradio.h"

#define PAYLOAD_LENGTH    2  // 1-15
#define FREQUENCY        15  // 0-83
#define SOP_PN_CODE       4  // 0-11
#define DATA_RATE        DATMODE_8DR

#define DAC_LOW_LIMIT    10
#define DAC_HIGH_LIMIT  140
#define OFF_THRESHOLD    10

void Setup_PSoC(void){
    M8C_EnableGInt;
    LCD_Start();
    PWM8_Start();
    DigBuf_Start(); 
    CMPPRG_Start(1);
    DAC8_Start(2);
    DAC8_WriteBlind(0x00); 
    ERROR_AMP_Start(1);
    PGA_BUF_Start(1);
}

void Setup_Radio(void){
    SPIM_Radio_Start(SPIM_Radio_SPIM_MODE_0 | SPIM_Radio_SPIM_MSB_FIRST); // Start the SPI module
    RadioInit(ACK_EN|END_STATE_SLEEP|ACK_TO_15X, DATCODE_LEN_32|DATMODE_8DR);
    // Data Transmission Modes: DATMODE_1MBPS, DATMODE_8DR, DATMODE_DDR, DATMODE_SDR     
    
    RadioSetFrequency(FREQUENCY);
    RadioSetSopPnCode(SOP_PN_CODE);
    RadioSetLength(PAYLOAD_LENGTH);    
}

void display(unsigned char rx_data, unsigned char mode, unsigned char rx_count, unsigned char rssi ){
    LCD_Position(0,0);
    LCD_PrCString("D=");
    LCD_PrHexByte(rx_data);

    LCD_Position(0,8);
    LCD_PrCString("M=");                
    LCD_PrHexByte(mode);

    LCD_Position(1,0);
    LCD_PrCString("C=");
    LCD_PrHexByte(rx_count);
    
    LCD_Position(1,8);
    LCD_PrCString("RSSI=");
    LCD_PrHexByte(rssi);
}

void update_dac(unsigned char data){
    
    if(data < DAC_LOW_LIMIT) data = 0;
    if(data > DAC_HIGH_LIMIT) data = DAC_HIGH_LIMIT;
    
    DAC8_WriteBlind(data);    

}

void main(){

    unsigned char RxPayloadBuffer[PAYLOAD_LENGTH];
    unsigned char rx_state=0, rx_packet_size, rx_data=0, mode=0, rx_count=0, rssi=0, rx_done_flag=0;
    
    Setup_PSoC();
    Setup_Radio();

    RadioSetPtr(RxPayloadBuffer);
    RadioStartReceive();
    
    while(1){

        rx_state = RadioGetReceiveState();
        if(rx_state & RADIO_COMPLETE){
            rx_packet_size = RadioEndReceive();       
            rx_count++;
            if(!(rx_state & RADIO_ERROR)){
                if(rx_packet_size == PAYLOAD_LENGTH){
                    rx_data = RxPayloadBuffer[0];
                    mode = RxPayloadBuffer[1];
                    rssi = (RadioGetRssi() & 0x1F);
                    rx_done_flag = 1;
                }
            }
            RadioStartReceive();            
        }
        
        if(rx_done_flag){
            if(rx_data < OFF_THRESHOLD)DigBuf_Stop();
            else DigBuf_Start();
            update_dac(rx_data);
            rx_done_flag = 0;
            RxSig_LED_Invert();      
        }
        
        display(rx_data, mode, rx_count, rssi);
        if(rssi>0x10) RSSI_LED_On();
        else RSSI_LED_Off();                     
    }
}