//-----------------------------------------------------------------------------
// PSoC RF Remote LED-LIght Demo TX
// 
// PSoCEval1 Connections:
//      P0.1    : Analog Input,   0V-Vdd range VR
//      P1.2    : Digital Input,  IRQ
//      P1.3    : Digital Output, nSS
//      P1.4    : Digital Output, SCLK
//      P1.5    : Digital Output, MOSI
//      P1.6    : Digital Input,  MISO
//      P1.7    : Digital Output, RST
//
// Build by Imagecraft C-comiler, PSoC Designer Version 4.4 with SP2
//-----------------------------------------------------------------------------

#include <m8c.h>                       
#include "PSoCAPI.h"
#include "PSoCgpioint.h"
#include "lpregs.h"
#include "lpradio.h"
#include "stdlib.h"

#define PAYLOAD_LENGTH    2  // 1-15
#define FREQUENCY        15  // 0-98
#define SOP_PN_CODE       4  // 0-11
#define DATA_RATE        DATMODE_8DR

#define ADC_RESOLUTION  256                    // ADC resolution
#define BG_SCALE        ((ADC_RESOLUTION)/64)  // BarGraph scale factor

void Setup_PSoC(void){
    M8C_EnableGInt;
    PGA_Start(3);             // Start the PGA module
    ADCINC_Start(3);          // Start the ADC module
    SleepTimer_Start();
    SleepTimer_EnableInt();     
    LCD_Start();              // Start the LCD module
    LCD_InitBG(LCD_SOLID_BG); // Initialize LCD Bargraph
    LCD_Position(0,0);
    LCD_PrCString("TX   MODE=");
}

void Setup_Radio(void){
    // Start SPI Module
    SPIM_Radio_Start(SPIM_Radio_SPIM_MODE_0 | SPIM_Radio_SPIM_MSB_FIRST);
    // Initialize Radio
    RadioInit(ACK_EN|END_STATE_SLEEP|ACK_TO_15X, PA_N10_DBM|DATCODE_LEN_32|DATMODE_8DR);
    
    // RF Power Amp settings: PA_N30_DBM, PA_N25_DBM, PA_N20_DBM, PA_N15_DBM, PA_N10_DBM, PA_N5_DBM, PA_0_DBM, PA_4_DBM
    // Data Transmission Modes: DATMODE_1MBPS, DATMODE_8DR, DATMODE_DDR, DATMODE_SDR 
    
    RadioSetFrequency(FREQUENCY);
    RadioSetSopPnCode(SOP_PN_CODE);
    RadioSetLength(PAYLOAD_LENGTH);
}

void main(){

    unsigned char TxPayloadBuffer[PAYLOAD_LENGTH] = {0x00};
    unsigned char adc_read=0, mode=0, bar_position=0;

    Setup_PSoC();
    Setup_Radio();
    RadioSetPtr(TxPayloadBuffer);
       
    while(1){
        // Read POT value
        ADCINC_GetSamples(1);   
        while(ADCINC_fIsDataAvailable() == 0); // Wait until ADC data ready
        adc_read = ADCINC_wClearFlagGetData(); // Get result
        
        // Load TX buffer with data
        TxPayloadBuffer[0] = adc_read;
        TxPayloadBuffer[1] = mode;
        
        // Start the transmit to send the packet
        RadioStartTransmit(0, PAYLOAD_LENGTH);
        
        // Wait for the transmit to return a complete status
        while(!(RadioGetTransmitState() & RADIO_COMPLETE));
        
        // Complete the transmit operation
        RadioEndTransmit();
        
        // Display the POT value on LCD
        LCD_Position(0,10);
        LCD_PrHexByte(mode);
        LCD_Position(1,0);
        LCD_PrHexByte(adc_read);
        bar_position = (BYTE)(adc_read/BG_SCALE);
        LCD_DrawBG(1, 3, 13, bar_position);
        
        LED_Invert();

        SleepTimer_SyncWait(32, SleepTimer_WAIT_RELOAD); //64Hz
        mode++;
    }
}